<?php
namespace Toffe;

use ManiaControl\ManiaControl;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Settings\SettingManager;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Admin\AuthenticationManager;


/**
 * ManiaControl SpecialWelcome Plugin
 * @author Toffe
 * 
 */
class SpecialWelcome implements CallbackListener, Plugin {
	/**
	 * Constants
	 */
	const PLUGIN_ID				= 58;
	const PLUGIN_VERSION		= 1.2;
	const PLUGIN_NAME			= 'SpecialWelcome';
	const PLUGIN_AUTHOR			= 'Toffe';

	const SETTING_SPECIALWELCOME_OP_T0			=	"Moderator Welcome 1";
	const SETTING_SPECIALWELCOME_OP_T1			=	"Moderator Welcome 2";

	const SETTING_SPECIALWELCOME_ADMIN_T0		=	"Admin Welcome 1";
	const SETTING_SPECIALWELCOME_ADMIN_T1		=	"Admin Welcome 2";

	const SETTING_SPECIALWELCOME_MASTERADMIN_T0	=	"MasterAdmin Welcome 1";
	const SETTING_SPECIALWELCOME_MASTERADMIN_T1	=	"MasterAdmin Welcome 2";

	const SETTING_SPECIALWELCOME_PLAYER_T0		=	"Player Welcome 1";
	const SETTING_SPECIALWELCOME_PLAYER_T1		=	"Player Welcome 2";
	const SETTING_SPECIALWELCOME_PLAYER_T2		=	"Player Welcome 3";
	const SETTING_SPECIALWELCOME_PLAYER_T3		=	"Player Welcome 4";


	/**
	 * Private properties
	 */
	/** @var maniaControl $maniaControl */
	// Init vars
	private $msg_op = array();
	private $msg_admin = array();
	private $msg_masteradmin = array();
	private $msg_player = array();
	private $maniaControl = null;

	/**
	 * Prepares the Plugin
	 *
	 * @param ManiaControl $maniaControl
	 * @return mixed
	 */
	public static function prepare(ManiaControl $maniaControl) {
		$maniaControl->settingManager->initSetting(get_class(), self::SETTING_SPECIALWELCOME_OP_T0, 'Welcome Moderator %s! Have fun moderating');
		$maniaControl->settingManager->initSetting(get_class(), self::SETTING_SPECIALWELCOME_OP_T1, 'Welcome Moderator %s! Have fun!');
		
		$maniaControl->settingManager->initSetting(get_class(), self::SETTING_SPECIALWELCOME_ADMIN_T0, 'Welcome %s, o wait, you are admin!');
		$maniaControl->settingManager->initSetting(get_class(), self::SETTING_SPECIALWELCOME_ADMIN_T1, 'Welcome admin %s. Keep your player quiet! :)');

		$maniaControl->settingManager->initSetting(get_class(), self::SETTING_SPECIALWELCOME_MASTERADMIN_T0, 'Welcome %s. Pssst, he is Master Admin!');
		$maniaControl->settingManager->initSetting(get_class(), self::SETTING_SPECIALWELCOME_MASTERADMIN_T1, 'Please Welcome Master Admin %s (aka Big Boss)');

		$maniaControl->settingManager->initSetting(get_class(), self::SETTING_SPECIALWELCOME_PLAYER_T0, 'Welcome %s, have fun on our server!');
		$maniaControl->settingManager->initSetting(get_class(), self::SETTING_SPECIALWELCOME_PLAYER_T1, 'Welcome %s, thanks for joining us.');
		$maniaControl->settingManager->initSetting(get_class(), self::SETTING_SPECIALWELCOME_PLAYER_T2, 'Welcome %s, don\'t kill me');
		$maniaControl->settingManager->initSetting(get_class(), self::SETTING_SPECIALWELCOME_PLAYER_T3, 'Welcome to the server, %s.');
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::load()
	 */
	public function load(ManiaControl $maniaControl) {
		// Set maniacontrol in local class
		$this->maniaControl = $maniaControl;
		
		// Push settings in array
		$this->pushArray();
		
		// Register callbacks
		$this->maniaControl->callbackManager->registerCallbackListener(PlayerManager::CB_PLAYERCONNECT, $this, 'handlePlayerConnect');
		$this->maniaControl->callbackManager->registerCallbackListener(SettingManager::CB_SETTING_CHANGED, $this, 'handleSettingChangedCallback');
		return true;
	}
	
	/**
	 * Handle SettingChanged callback
	 *
	 * @param Setting $setting
	 */
	public function handleSettingChangedCallback(Setting $setting) {
		// Is it for me? nom nom nom
		if (!$setting->belongsToClass($this)) return;
		
		$this->pushArray();
		$this->maniaControl->chat->sendInformation("SpecialWelcome settings saved! ", null, false);
	}

	/**
	 * Handle PlayerConnect callback
	 *
	 * @param Player $player
	 */
	public function handlePlayerConnect(Player $player) {
		$formatstring = "";

		switch ($player->authLevel) {
			case AuthenticationManager::AUTH_LEVEL_PLAYER:
				$r = rand(0,count($this->msg_player)-1);
				$formatstring = (isset($this->msg_player[$r]) ? $this->msg_player[$r] : 'Welcome %s!!!');
				break;
			case AuthenticationManager::AUTH_LEVEL_MODERATOR:
				$r = rand(0,count($this->msg_op)-1);
				$formatstring = (isset($this->msg_op[$r]) ? $this->msg_op[$r] : 'Welcome %s!!!');
				break;
			case AuthenticationManager::AUTH_LEVEL_ADMIN:
				$r = rand(0,count($this->msg_admin)-1);
				$formatstring = (isset($this->msg_admin[$r]) ? $this->msg_admin[$r] : 'Welcome %s!!!');
				break;
			case AuthenticationManager::AUTH_LEVEL_SUPERADMIN:
			case AuthenticationManager::AUTH_LEVEL_MASTERADMIN:
				$r = rand(0,count($this->msg_masteradmin)-1);
				$formatstring = (isset($this->msg_masteradmin[$r]) ? $this->msg_masteradmin[$r] : 'Welcome %s!!!');
				break;
			default:
				return;
				break;
		}

		if(stristr($formatstring, "%s") == false) return;
		
		$chatmsg = '$ff3'.sprintf($formatstring, $player->nickname.'$z$ff3');
		$this->maniaControl->chat->sendChat($chatmsg, null, true);
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::unload()
	 */
	public function unload() {
		//
	}

	public function pushArray()
	{
		// Clean (outdated)array
		$this->msg_op = array();
		$this->msg_admin = array();
		$this->msg_masteradmin = array();
		$this->msg_player = array();

		// Walk in const (like a array but it isn't :)  )
		for ($i=0; $i <= 1; $i++) { 
			if($this->maniaControl->settingManager->getSetting($this, constant("self::SETTING_SPECIALWELCOME_OP_T{$i}")) != "") $this->msg_op[] = $this->maniaControl->settingManager->getSetting($this, constant("self::SETTING_SPECIALWELCOME_OP_T$i"));
		}

		for ($i=0; $i <= 1; $i++) { 
			if($this->maniaControl->settingManager->getSetting($this, constant("self::SETTING_SPECIALWELCOME_ADMIN_T{$i}")) != "") $this->msg_admin[] = $this->maniaControl->settingManager->getSetting($this, constant("self::SETTING_SPECIALWELCOME_ADMIN_T$i"));
		}

		for ($i=0; $i <= 1; $i++) { 
			if($this->maniaControl->settingManager->getSetting($this, constant("self::SETTING_SPECIALWELCOME_MASTERADMIN_T{$i}")) != "") $this->msg_masteradmin[] = $this->maniaControl->settingManager->getSetting($this, constant("self::SETTING_SPECIALWELCOME_MASTERADMIN_T$i"));
		}

		for ($i=0; $i <= 3; $i++) { 
			if($this->maniaControl->settingManager->getSetting($this, constant("self::SETTING_SPECIALWELCOME_PLAYER_T{$i}")) != "") $this->msg_player[] = $this->maniaControl->settingManager->getSetting($this, constant("self::SETTING_SPECIALWELCOME_PLAYER_T$i"));
		}
	}

	
	/**
	 * @see \ManiaControl\Plugins\Plugin::getId()
	 */
	public static function getId() {
		return self::PLUGIN_ID;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getName()
	 */
	public static function getName() {
		return self::PLUGIN_NAME;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getVersion()
	 */
	public static function getVersion() {
		return self::PLUGIN_VERSION;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getAuthor()
	 */
	public static function getAuthor() {
		return self::PLUGIN_AUTHOR;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getDescription()
	 */
	public static function getDescription() {
		return 'Welcomes every player and Admin/Moderator with a few customizable messages.';
	}


}
